/*****************************************************************************
 * $LastChangedDate: 2009-11-22 21:39:11 -0600 (Sun, 22 Nov 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Module configuration.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef BASE_CONF_HH
#define BASE_CONF_HH 1

#include "base/conf_txt.hh"
#include "base/time.hh"

namespace base {

////////////////////////////////////////////////////////////////////////////////
/// @brief Module configuration.
///
class Conf
{
private:
    ////////////////////////////////////////////////////////////////////////////
    /// @brief Template Method to parse fields in textual .conf file.
    ///
    class ModuleConfTxt : public ConfTxt
    {
    public:
        void ProcessField( string& field, ifstream& conf );
    };

public:
            Conf( void );
    void    Init( int argc, char** argv, const string& programName, const string& confTxtFilename );

private:
    void    ParseCmdlineArgs( int argc, char** argv );
    void    ParseCmdlineArgs2( int argc, char** argv );
    void    ReadConfTxt( const string& confTxtFilename );

public:
    bool    mValid;             ///< true if init/read

    string  mProgramName;       ///< determines full pathname of data dir

    // Cmd-line args:
    int     mArgc;
    char**  mArgv;
    string  mArgv0;
    bool    mArgVerbose;
    bool    mArgQuiet;
#if OS_UNIX
    bool    mTimerUnixGettimeofday;     ///< TimeClock: -time-gettimeofday
#endif

    // .conf:
    bool            mVerbose;
    bool            mQuiet;
    uint            mRandomSeed;
    Milliseconds    mTimeClockSkewMilliseconds;     ///< TimeClock: time delta threshold when calendar-clock is considered skewed
};

ostream& operator<<( ostream& strm, const Conf& conf );

//------------------------------------------------------------------------------

// NOTE: "global" is a namespace, "gGlobals" is a var (instance of Globals class).

namespace global { BASE_GLOBAL Conf gConf; }

// BASE_CONF is an abbrevation and also ensures configuration was read in debug build.
#if DEBUG
INLINE Conf& GetConf( void ) { ASSERT(base::global::gConf.mValid);
                               return base::global::gConf; }
#define BASE_CONF base::GetConf()
#else
#define BASE_CONF base::global::gConf
#endif

//------------------------------------------------------------------------------

const string&
GetDataDir( void );

} // namespace base

#endif // BASE_CONF_HH
