/******************************************************************************
 * $LastChangedDate: 2009-11-22 21:39:11 -0600 (Sun, 22 Nov 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Debug support (unused because my code is always perfect).
 *//*
 * LEGAL:   COPYRIGHT (C) 2006 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 ******************************************************************************/

#if DEBUG

#define BASE_DEBUG_CC 1
#include "base/module.hh"
#include "base/debug.hh"

#if ! OS_WINDOWS
#include <sstream>
#include <sys/time.h>
#include <time.h>
#if __GLIBC__
#include <malloc.h>
#endif
#endif // ! OS_WINDOWS

namespace base {

#if DEBUG
// For a check when compiling tests which require DEBUG build.
bool BASE_DEBUG_BUILD = true;
#endif

/*****************************************************************************
 * Return total amount of bytes allocated.
 *****************************************************************************/
unsigned long
MemUsed( void )
{
#if __GLIBC__
    // Per delorie.com:
    // Example:
    // struct mallinfo info = mallinfo();
    // printf("Memory in use: %d bytes\n", info.usmblks + info.uordblks);
    // printf("Total heap size: %d bytes\n", info.arena);

    struct mallinfo meminfo;
    meminfo = mallinfo();
  //return meminfo.arena;
    return meminfo.usmblks + meminfo.uordblks;
#else
    return 0;  // yeh right
#endif
}

/*****************************************************************************
 * Measure the amount of time a function takes.
 *****************************************************************************/
#if ! OS_WINDOWS
void
MeasureTime( void(*func)(long), long funcArg, const string& funcName )
{
    // ANSI C code that measures the time the process consumed in user-mode.
    // Excludes kernel-mode (doesn't count time spent in video driver).
    int clocks1 = clock();
    func( funcArg );  // call function to be measured
    int clocks2      = clock();
    int clocksPerSec = CLOCKS_PER_SEC;
    fp  secUser      = fp(clocks2 - clocks1) / fp(clocksPerSec);
    ostringstream oss;
    oss << "------------------------------------\n"
        << "function: " << funcName << "\n"
        << "secs (user) = " << secUser << "\n"
        << "------------------------------------\n";
    string result = oss.str();
    CDEBUG << result.c_str();
}
#endif // ! OS_WINDOWS

} // namespace base

#endif // DEBUG
