/*****************************************************************************
 * $LastChangedDate: 2009-12-16 20:42:17 -0600 (Wed, 16 Dec 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   control module configuration.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef CONTROL_CONF_HH
#define CONTROL_CONF_HH 1

#include "base/time.hh"
#include "base/conf_txt.hh"
#include "math/module.hh"
using namespace math;
#include "physics/types.hh"
using namespace physics;

namespace control {

////////////////////////////////////////////////////////////////////////////////
/// @brief control module configuration.
///
class Conf
{
private:
    ////////////////////////////////////////////////////////////////////////////
    /// @brief Template Method to parse fields in .conf file.
    ///
    class ModuleConfTxt : public base::ConfTxt
    {
    public:
        void ProcessField( string& field, ifstream& conf );
    };

public:
            Conf( void );
    void    Init( int argc, char** argv, const string& confTxtFilename );

private:
    void    ParseCmdlineArgs( int argc, char** argv );
    void    ParseCmdlineArgs2( int argc, char** argv );
    void    ReadConfTxt( const string& confTxtFilename );
    void    ReadJoystickConfTxt( void );

public:
    bool            mValid;             ///< true if Conf was init/read

    // Cmd-line args:
    bool            mJoystickEnabled;
    bool            mJoystickAxis3;
    bool            mJoystickPrint;
    string          mJoystickConfFilename;
};

ostream& operator<<( ostream& strm, const Conf& conf );

//------------------------------------------------------------------------------

// NOTE: "global" is a namespace, "gGlobals" is a var (instance of Globals class).

namespace global {
#if CONTROL_MODULE_CC
Conf gConf;
#else
extern Conf gConf;
#endif
}  // namespace global

// CONTROL_CONF is an abbrevation and also ensures configuration was read in debug build.
#if DEBUG
INLINE Conf& GetConf( void ) { ASSERT(control::global::gConf.mValid);
                               return control::global::gConf; }
#define CONTROL_CONF control::GetConf()
#else
#define CONTROL_CONF control::global::gConf
#endif

//------------------------------------------------------------------------------

} // namespace control

#endif // CONTROL_CONF_HH
