/*****************************************************************************
 * $LastChangedDate: 2009-11-22 21:39:11 -0600 (Sun, 22 Nov 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Special-effects container.
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef FX_FX_HH
#define FX_FX_HH 1

#define GET_FX() (fx::FX::GetInstance())

#include "base/timer.hh"

namespace fx {

class ParticleSystem;

////////////////////////////////////////////////////////////////////////////////
/// @brief Special-effects container (Singleton).
///
class FX
{
PREVENT_COPYING(FX)
private:
    // Containers of special-effects.
    typedef list<shptr<Object> > Objects;
    typedef list<shptr<ParticleSystem> > ParticleSystems;

    FX( void );
    ~FX();

private:
    ////////////////////////////////////////////////////////////////////////////
    /// @brief Timer-tick.
    ///
    class TimerFunctor : public Timer::Functor
    {
    public:
        TimerFunctor( FX& fx ) : mFX(fx) { }
        void operator()( const Milliseconds millisecElapsed );
    private:
        FX& mFX;
    };

// Interface for clients:
public:
                DEF_GetInstance( FX )   // Singleton
    void        Reset( void );
    void        MakeExplosion( const Milliseconds lifetime,
                               const WorldVertex& position,
                               const fp radius );
    void        MakeSmokeTrail( const Milliseconds lifetime,
                                shptr<Object> trailMaker );

// Restricted to fx module:
public:
    void        AddZombie( shptr<Object> zombie );
    void        AddZombie( shptr<ParticleSystem> zombie );

// Private methods:
private:
    void        Tick( const Milliseconds millisecElapsed );
    void        Detach( Objects& objects );  // plural
    void        Detach( ParticleSystems& particleSystems );  // plural

private:
    CLASS_VAR FX*       msInstance;
    shptr<TimerFunctor> mTimerFunctor;          ///< tick for destroying zombies
    Objects             mObjects;               ///< container of active special-effects
    Objects             mZombieObjects;         ///< AddZombie() adds to this
    ParticleSystems     mParticleSystems;       ///< container of active special-effects
    ParticleSystems     mZombieParticleSystems; ///< AddZombie() adds to this
};

} // namespace fx

#endif // FX_FX_HH
