/*****************************************************************************
 * $LastChangedDate: 2009-12-14 19:05:10 -0600 (Mon, 14 Dec 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   glue module configuration.
 * @remarks glue's configuration is degenerate from the other modules.
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef GLUE_CONF_HH
#define GLUE_CONF_HH 1

namespace glue {

string
GetHomeDirectoryName( void );

string
GetConfDirectoryName( void );

////////////////////////////////////////////////////////////////////////////////
/// @brief glue module configuration.
///
/// This is degenerate.
///
class Conf
{
public:
            Conf( void );
    void    Init( int argc, char** argv,
                  const string& confTxtFilename, const string& programName );

private:
    void    ParseCmdlineArgs( int argc, char** argv );

public:
    bool            mValid;             ///< true if Conf was init/read
    string          mProgramName;       ///< argv[0]

    // Cmd-line args:
    bool            mFast;              ///< try to run fast on slow computers
#if COMPILE_SDL
    bool            mEnableSDL;
#endif
};

//------------------------------------------------------------------------------

// NOTE: "global" is a namespace, "gGlobals" is a var (instance of Globals class).

namespace global {
#if GLUE_CONF_CC
Conf gConf;
#else
extern Conf gConf;
#endif
}  // namespace global

// GLUE_CONF is an abbrevation and also ensures configuration was read in debug build.
#if DEBUG
INLINE Conf& GetConf( void ) { ASSERT(glue::global::gConf.mValid);
                               return glue::global::gConf; }
#define GLUE_CONF glue::GetConf()
#else
#define GLUE_CONF glue::global::gConf
#endif

//------------------------------------------------------------------------------

} // namespace glue

#endif // GLUE_CONF_HH
