/*****************************************************************************
 * $LastChangedDate: 2010-02-18 12:32:07 -0600 (Thu, 18 Feb 2010) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Module configuration.
 *//*
 * LEGAL:   COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef GRAPH_CONF_HH
#define GRAPH_CONF_HH 1

#include "base/conf_txt.hh"
#include "gfx/rgba.hh"
using namespace gfx;

namespace graph {

////////////////////////////////////////////////////////////////////////////////
/// @brief Module configuration.
///
class Conf
{
private:
    ////////////////////////////////////////////////////////////////////////////
    /// @brief Template Method to parse fields in .conf file.
    ///
    class ModuleConfTxt : public base::ConfTxt
    {
    public:
        void ProcessField( string& field, ifstream& conf );
    };

public:
            Conf( void );
    void    Init( int argc, char** argv, const string& confTxtFilename );

private:
    void    ParseCmdlineArgs( int argc, char** argv );
    void    ParseCmdlineArgs2( int argc, char** argv );
    void    ReadConfTxt( const string& confTxtFilename );

public:
    bool            mValid;                 ///< true if Conf was init/read
    // cmdline:
    bool            mEnableOsgOptimizer;    ///< disabled by -no-osg-opt
    bool            mEnableWireframe;       ///< -wire,-wireframe or Mod-F9 key
    bool            mEnableDumpFull;        ///< when scene-graph is dumped, dump as much as possible (wastes memory)
    bool            mEnableFog;
    int             mShadowMode;            ///< disabled if equals defs::SHADOW_MODE_DISABLED
    // conf file:
    uint            mShadowLightNum;
    osg::Vec4       mShadowLightPos;
    osg::Vec3       mShadowLightDir;
    RGBA            mShadowLightColor;
    RGBA            mBackgroundColor;       ///< clear color
};

ostream& operator<<( ostream& strm, const Conf& conf );

//------------------------------------------------------------------------------

// NOTE: "global" is a namespace, "gGlobals" is a var (instance of Globals class).

namespace global {
#if GRAPH_CONF_CC
Conf gConf;
#else
extern Conf gConf;
#endif
}  // namespace global

// GRAPH_CONF is an abbrevation and also ensures configuration was read in debug build.
#if DEBUG
INLINE Conf& GetConf( void ) { ASSERT(graph::global::gConf.mValid);
                               return graph::global::gConf; }
#define GRAPH_CONF graph::GetConf()
#else
#define GRAPH_CONF graph::global::gConf
#endif

//------------------------------------------------------------------------------

} // namespace graph

#endif // GRAPH_CONF_HH
