/*****************************************************************************
 * $LastChangedDate: 2009-11-22 21:39:11 -0600 (Sun, 22 Nov 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   HUD module configuration.
 *//*
 * LEGAL:   COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define HUD_CONF_CC 1
#include "base/module.hh"
#include "base/conf.hh"
#include "base/misc.hh"
using namespace base;
#include "hud/module.hh"
#include "hud/defs.hh"
#include "hud/conf.hh"
#include "hud/font.hh"

namespace hud {

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////  Conf  ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * ctor/dtor.
 *****************************************************************************/
Conf::Conf( void )
:   mValid(false)
    // cmd-line args 
    // .conf file
{
}

/*****************************************************************************
 * Initialize/read configuration.
 *****************************************************************************/
void
Conf::Init( int argc, char** argv, const string& confTxtFilename )
{

BEGIN_ONCE {

    mValid = true;  // assign this first as *_CONF may expand to GetConf() which asserts it

    ParseCmdlineArgs( argc, argv );   // cmdline args parsed prior to reading .conf file
    ReadConfTxt( confTxtFilename );   // read from textual .conf file
    ParseCmdlineArgs2( argc, argv );  // cmdline args that override .conf file

} END_ONCE

}

/*****************************************************************************
 * Parse cmdline args.
 *****************************************************************************/
void
Conf::ParseCmdlineArgs( int argc, char** argv )
{
    // NOP
}

void
Conf::ParseCmdlineArgs2( int argc, char** argv )
{
    // NOP
}

/*****************************************************************************
 * Read from .conf file.
 *****************************************************************************/
void
Conf::ReadConfTxt( const string& confTxtFilename )
{
    try
    {
        ModuleConfTxt confTxt;
        confTxt.Read( string(base::GetDataDir()) + string(confTxtFilename) );
    }
    catch (...)
    {
        CERROR << "ERROR: cannot read " << confTxtFilename << endl;
        EXIT( 1 );
    }
}

void
Conf::ModuleConfTxt::ProcessField( string& field, ifstream& conf )
{
    bool error = false;

    if ( field == "font" )
    {
        // Defer further parsing remaining fields to FontMap.
        error = not GET_FONT_MAP().ParseFont( conf );
    }

    // Errors?
    if ( error )
    {
        CERROR << "ERROR[conf]: parameter for " << field << " is invalid." << endl;
        EXIT( 1 );
    }
}

/*****************************************************************************
 * Print Conf configuration.
 *****************************************************************************/
ostream&
operator<<( ostream& strm, const Conf& conf )
{
    return strm;
}

} // namespace hud
