/*****************************************************************************
 * $LastChangedDate: 2009-11-22 21:39:11 -0600 (Sun, 22 Nov 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   HUD module configuration.
 *//*
 * LEGAL:   COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef HUD_CONF_HH
#define HUD_CONF_HH 1

#include "base/module.hh"
#include "base/conf_txt.hh"
using namespace base;

namespace hud {

////////////////////////////////////////////////////////////////////////////////
/// @brief Module configuration.
///
class Conf
{
private:
    ////////////////////////////////////////////////////////////////////////////
    /// @brief Template Method to parse fields in .conf file.
    ///
    class ModuleConfTxt : public base::ConfTxt
    {
    public:
        void ProcessField( string& field, ifstream& conf );
    };

public:
            Conf( void );
    void    Init( int argc, char** argv, const string& confFilename );

private:
    void    ParseCmdlineArgs( int argc, char** argv );
    void    ParseCmdlineArgs2( int argc, char** argv );
    void    ReadConfTxt( const string& confTxtFilename );

public:
    bool                mValid;     ///< true if Conf was init/read

    // Cmd-line args:

    // .conf:
};

ostream& operator<<( ostream& strm, const Conf& conf );

//------------------------------------------------------------------------------

// NOTE: "global" is a namespace, "gGlobals" is a var (instance of Globals class).

namespace global {
#if HUD_MODULE_CC
Conf gConf;
#else
extern Conf gConf;
#endif
}  // namespace global

// HUD_CONF is an abbrevation and also ensures configuration was read in debug build.
#if DEBUG
INLINE Conf& GetConf( void ) { ASSERT(hud::global::gConf.mValid);
                               return hud::global::gConf; }
#define HUD_CONF hud::GetConf()
#else
#define HUD_CONF hud::global::gConf
#endif

//------------------------------------------------------------------------------

} // namespace hud

#endif // HUD_CONF_HH
