/*****************************************************************************
 * $LastChangedDate: 2009-12-14 19:05:10 -0600 (Mon, 14 Dec 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Module configuration.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef INPUT_CONF_HH
#define INPUT_CONF_HH 1

#include "base/conf_txt.hh"

namespace input {

////////////////////////////////////////////////////////////////////////////////
/// @brief Module configuration.
///
class Conf
{
private:
    ////////////////////////////////////////////////////////////////////////////
    /// @brief Template Method to parse fields in .conf file.
    ///
    class ModuleConfTxt : public base::ConfTxt
    {
    public:
        void ProcessField( string& field, ifstream& conf );
    };

public:
            Conf( void );
    void    Init( int argc, char** argv, const string& confFilename );

private:
    void    ParseCmdlineArgs( int argc, char** argv );
    void    ParseCmdlineArgs2( int argc, char** argv );
    void    ReadConfTxt( const string& confTxtFilename );

public:
    bool            mValid;     ///< true if Conf was init/read

    // Cmd-line args:

    // .conf:
    Milliseconds    mInputQueueTickFreq;
    uint            mKeyboardInputQueueOverflow;
    uint            mJoystickInputQueueOverflow;
    uint            mMouseInputQueueOverflow;
    uint            mJoystickNum;               ///< joystick number (if applicable)
    bool            mEnableLinuxJoystick;       ///< true means user has requested Linux joystick driver
    bool            mEnableWindowsJoystick;     ///< true means user has requested Windows joystick driver
};

ostream& operator<<( ostream& strm, const Conf& conf );

//------------------------------------------------------------------------------

// NOTE: "global" is a namespace, "gGlobals" is a var (instance of Globals class).

namespace global {
#if INPUT_MODULE_CC
Conf gConf;
#else
extern Conf gConf;
#endif
}  // namespace global

// INPUT_CONF is an abbrevation and also ensures configuration was read in debug build.
#if DEBUG
INLINE Conf& GetConf( void ) { ASSERT(input::global::gConf.mValid);
                               return input::global::gConf; }
#define INPUT_CONF input::GetConf()
#else
#define INPUT_CONF input::global::gConf
#endif

//------------------------------------------------------------------------------

} // namespace input

#endif // INPUT_CONF_HH
