/*****************************************************************************
 * $LastChangedDate: 2009-11-22 21:39:11 -0600 (Sun, 22 Nov 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Interface of C functions for Lua scripts.
 * @remarks @remarks module.txt documents how to export new C++ classes to Lua.
 *
 * ----------------------------
 * -- THIS IS AUTO-GENERATED --
 * ----------------------------
 *
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#include "base/module.hh"
using namespace base;
#include "physics/module.hh"
#include "physics/physics_control.hh"
using namespace physics;
#include "lua_bind/module.hh"
#include "lua_bind/bind.hh"

namespace lua {


namespace bind {

#define BIND_CLASS_NAME      "Physics"
#define BIND_METATABLE_NAME  LUABIND_METATABLE_STRING(BIND_CLASS_NAME)

//==============================================================================
// C functions called by Lua that access C++ objects.
//==============================================================================

/*****************************************************************************
 * Physics:Enable()
 *****************************************************************************/
INTERN int 
Physics_Enable( lua_State* L ) 
{ 
   try { 

        PhysicsControl::Enable( POP_BOOL( L ) );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Physics:EnableStall( bool )
 *****************************************************************************/
INTERN int 
Physics_EnableStall( lua_State* L ) 
{ 
   try { 

        PhysicsControl::EnableStall( POP_BOOL( L ) );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * bool Physics:IfEnabled()
 *****************************************************************************/
INTERN int 
Physics_IfEnabled( lua_State* L ) 
{ 
   try { 

        lua_pushboolean( L, PhysicsControl::IfEnabled() );
        return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * bool Physics:IfStallEnabled()
 *****************************************************************************/
INTERN int 
Physics_IfStallEnabled( lua_State* L ) 
{ 
   try { 

        lua_pushboolean( L, PhysicsControl::IfStallEnabled() );
        return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Speed Physics:KPH2Speed( kph )
 *****************************************************************************/
INTERN int 
Physics_KPH2Speed( lua_State* L ) 
{ 
   try { 

        const SpeedKPH kph( POP_FP( L ) );
        const Speed    speed( physics::conv::KPH2Speed( kph ) );
        lua_pushnumber( L, speed.FP() );
        return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Physics:PermanentlyDisable()
 *****************************************************************************/
INTERN int 
Physics_PermanentlyDisable( lua_State* L ) 
{ 
   try { 

        PhysicsControl::PermanentlyDisable();
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * KPH Physics:Speed2KPH( Speed )
 *****************************************************************************/
INTERN int 
Physics_Speed2KPH( lua_State* L ) 
{ 
   try { 

        const Speed    speed( POP_FP( L ) );
        const SpeedKPH speedKPH( physics::conv::Speed2KPH( speed ) );
        lua_pushnumber( L, speedKPH.FP() );
        return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

//==============================================================================
// Register C functions with Lua. 
//==============================================================================

#if ! DOXYGEN 

INTERN const struct luaL_reg sMethodTableNew[] =
{
  // new,__gc omitted as Lua cannot instantiate this C++ class.
  { NULL, NULL }
};

INTERN const struct luaL_reg sMethodTable[] =
{
  // __gc omitted as Lua cannot instantiate this C++ class.
  { "Enable", Physics_Enable },
  { "EnableStall", Physics_EnableStall },
  { "IfEnabled", Physics_IfEnabled },
  { "IfStallEnabled", Physics_IfStallEnabled },
  { "KPH2Speed", Physics_KPH2Speed },
  { "PermanentlyDisable", Physics_PermanentlyDisable },
  { "Speed2KPH", Physics_Speed2KPH },
  { NULL, NULL }
};

void
RegisterPhysics( lua_State* L )
{
    REGISTER_CLASS_METATABLE( L,
                              BIND_METATABLE_NAME,
                              BIND_CLASS_NAME,
                              sMethodTableNew,
                              sMethodTable );
}

#endif // ! DOXYGEN 

} // namespace bind 

} // namespace lua 
