/*****************************************************************************
 * $LastChangedDate: 2009-11-22 21:39:11 -0600 (Sun, 22 Nov 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Interface of C functions for Lua scripts.
 * @remarks @remarks module.txt documents how to export new C++ classes to Lua.
 *
 * ----------------------------
 * -- THIS IS AUTO-GENERATED --
 * ----------------------------
 *
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#include "base/module.hh"
using namespace base;
#include "program/module.hh"
using namespace program;
#include "lua_bind/module.hh"
#include "lua_bind/bind.hh"

namespace lua {


namespace bind {

#define BIND_CLASS_NAME      "Program"
#define BIND_METATABLE_NAME  LUABIND_METATABLE_STRING(BIND_CLASS_NAME)

//==============================================================================
// C functions called by Lua that access C++ objects.
//==============================================================================

/*****************************************************************************
 * string Program:DequeueCommand()
 * Returns next command or nil.
 *****************************************************************************/
INTERN int 
Program_DequeueCommand( lua_State* L ) 
{ 
   try { 

        const string cmd = GET_PROGRAM().DequeueCommand();
        if ( cmd.empty() )
            lua_pushnil( L );
        else
            lua_pushstring( L, cmd.c_str() );
        return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Program:Exit( int )
 *****************************************************************************/
INTERN int 
Program_Exit( lua_State* L ) 
{ 
   try { 

        EXIT( POP_INT( L ) );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Program:FlushSceneGraph() -- flushes then reloads essential parts of scene-graph
 *****************************************************************************/
INTERN int 
Program_FlushSceneGraph( lua_State* L ) 
{ 
   try { 

        GET_PROGRAM().FlushSceneGraph();
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * bool Program:IfPaused()
 *****************************************************************************/
INTERN int 
Program_IfPaused( lua_State* L ) 
{ 
   try { 

        lua_pushboolean( L, GET_PROGRAM().IfPaused() );
        return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Program:Reset()
 *****************************************************************************/
INTERN int 
Program_Reset( lua_State* L ) 
{ 
   try { 

        GET_PROGRAM().Reset();
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Program:SetPause( bool )
 *****************************************************************************/
INTERN int 
Program_SetPause( lua_State* L ) 
{ 
   try { 

        GET_PROGRAM().SetPause( POP_BOOL( L ) );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

//==============================================================================
// Register C functions with Lua. 
//==============================================================================

#if ! DOXYGEN 

INTERN const struct luaL_reg sMethodTableNew[] =
{
  // new,__gc omitted as Lua cannot instantiate this C++ class.
  { NULL, NULL }
};

INTERN const struct luaL_reg sMethodTable[] =
{
  // __gc omitted as Lua cannot instantiate this C++ class.
  { "DequeueCommand", Program_DequeueCommand },
  { "Exit", Program_Exit },
  { "FlushSceneGraph", Program_FlushSceneGraph },
  { "IfPaused", Program_IfPaused },
  { "Reset", Program_Reset },
  { "SetPause", Program_SetPause },
  { NULL, NULL }
};

void
RegisterProgram( lua_State* L )
{
    REGISTER_CLASS_METATABLE( L,
                              BIND_METATABLE_NAME,
                              BIND_CLASS_NAME,
                              sMethodTableNew,
                              sMethodTable );
}

#endif // ! DOXYGEN 

} // namespace bind 

} // namespace lua 
