/*****************************************************************************
 * $LastChangedDate: 2009-11-22 21:39:11 -0600 (Sun, 22 Nov 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Macros/functions to assist and simplify writing glue code between C++ and Lua.
 * @remarks Indeed, macros.cc is a C++ file.
 *------------------------------------------------------------------------------
 * REMEMBER: Indexs of tables passed between C++ and Lua must be ## ZERO-BASED ##.
 * Lua scripts must define special Lua classes such as Vec3 with zero-based indexs
 * because passing array-like one-based Lua tables to zero-based C++ is wrong.
 *------------------------------------------------------------------------------
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

// Avoid lua_/LUA_ which is reserved by Lua.

#define LUABIND_MACROS_GENERIC_CC 1
#include "base/module.hh"
using namespace base;
#include "lua_bind/module.hh"
#include "lua_bind/bind.hh"

namespace lua {
namespace bind {

/*****************************************************************************
 * Register a metatable to emulate a class in Lua.
 *****************************************************************************/
void
REGISTER_CLASS_METATABLE( lua_State* L,
                          const char* metaTableName,
                          const char* className,
                          const struct luaL_reg methodTableNew[],
                          const struct luaL_reg methodTable[] )
{
SAVE_STACK(L);

#if 1
    // Once only, make a global table for this C module.
    // It emulates a namespace inside Lua.
    lua_getglobal( L, LUABIND_MODULE_NAME );
    if ( lua_isnil( L, -1 ) )
    {
        lua_newtable( L );
        lua_setglobal( L, LUABIND_MODULE_NAME );
    }
    else
    {
        // Already exists.
        lua_pop( L, 1 );
    }

    // Then make a sub-table for this class in this C module's table.
    lua_getglobal( L, LUABIND_MODULE_NAME );
    lua_newtable( L );
    lua_setfield( L, -2, className );

    // Make a Lua metatable for this class.
    luaL_newmetatable( L, metaTableName );
    lua_pushstring( L, "__index" );
    lua_pushvalue( L, -2 );     // pushes the metatable
    lua_settable( L, -3 );      // metatable.__index = metatable
    luaL_register( L, NULL, methodTable );
    lua_getglobal( L, LUABIND_MODULE_NAME );
    if ( lua_type( L, -1 ) != LUA_TTABLE )
        THROW_LUA_EXCEPTION( "REGISTER_CLASS_METATABLE: lua_getglobal(module)" );
    lua_getfield( L, -1, className );
    if ( lua_type( L, -1 ) != LUA_TTABLE )
        THROW_LUA_EXCEPTION( "REGISTER_CLASS_METATABLE: lua_getfield(className)" );
    luaL_register( L, NULL, methodTableNew );

    // And register the methods with the table "sim.CLASS" itself.
    // This is similar to C++ "class methods".
    lua_getglobal( L, LUABIND_MODULE_NAME );
    lua_getfield( L, -1, className );
    luaL_register( L, NULL, methodTable );

#else // old way without using a namespace

    luaL_newmetatable( L, metaTableName );
    lua_pushstring( L, "__index" );
    lua_pushvalue( L, -2 );     // pushes the metatable
    lua_settable( L, -3 );      // metatable.__index = metatable
    luaL_register( L, NULL,      methodTable );
    luaL_register( L, className, methodTableNew );

#endif

RESTORE_STACK(L);
}

} // namespace bind
} // namespace lua
