/*****************************************************************************
 * $LastChangedDate: 2009-11-22 21:39:11 -0600 (Sun, 22 Nov 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Functions to convert physical metrics.
 * @remarks Complements world/conversion.hh
 * @remarks physics/conversion.hh converts between different physical units.
 *          world/conversion.hh converts between graphical coordinates.
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef PHYSICS_CONVERSION_HH
#define PHYSICS_CONVERSION_HH 1

#include "base/time.hh"

namespace physics {
namespace conv {

SpeedMPH
KPH2MPH( const SpeedKPH kph );

Speed
KPH2Speed( const SpeedKPH kph );

fp
KPH2Step( const Milliseconds milliseconds, const SpeedKPH kph );

SpeedKPH
Speed2KPH( const Speed speed );

SpeedMPH
Speed2MPH( const Speed speed );

SpeedMPH
Mach2MPH( const Mach mach );

SpeedKPH
Mach2KPH( const Mach mach );

Speed
Mach2Speed( const Mach mach );

Mach
Speed2Mach( const Speed speed );

} // namespace conv
} // namespace physics

#endif // PHYSICS_CONVERSION_HH
