/*****************************************************************************
 * $LastChangedDate: 2010-01-02 10:38:46 -0600 (Sat, 02 Jan 2010) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Module configuration.
 *//*
 * LEGAL:   COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef SHADER_CONF_HH
#define SHADER_CONF_HH 1

#include <osg/Fog>
#include "base/conf_txt.hh"
#include "gfx/module.hh"
#include "gfx/rgba.hh"
using namespace gfx;

namespace shader {

////////////////////////////////////////////////////////////////////////////////
/// @brief Module configuration.
///
class Conf
{
private:
    ////////////////////////////////////////////////////////////////////////////
    /// @brief Template Method to parse fields in .conf file.
    ///
    class ModuleConfTxt : public base::ConfTxt
    {
    public:
        void ProcessField( string& field, ifstream& conf );
    };

public:
            Conf( void );
    void    Init( int argc, char** argv, const string& confTxtFilename );

private:
    void    ParseCmdlineArgs( int argc, char** argv );
    void    ParseCmdlineArgs2( int argc, char** argv );
    void    ReadConfTxt( const string& confTxtFilename );

public:
    bool            mValid;                 ///< true if Conf was init/read
    bool            mEnableShaders;         ///< enable shaders
    bool            mEnableLighting;        ///< enable lighting
    osg::Vec3       mLight0Position;
    bool            mEnableFog;
    RGBA            mFogColor;
    fpx             mFogDensity;
    osg::Fog::Mode  mFogMode;
    fp              mFogMin;                ///< unit range (isn't a density)
    fp              mBrightness;
};

ostream& operator<<( ostream& strm, const Conf& conf );

//------------------------------------------------------------------------------

// NOTE: "global" is a namespace, "gGlobals" is a var (instance of Globals class).

namespace global {
#if SHADER_CONF_CC
Conf gConf;
#else
extern Conf gConf;
#endif
}  // namespace global

// SHADER_CONF is an abbrevation and also ensures configuration was read in debug build.
#if DEBUG
INLINE Conf& GetConf( void ) { ASSERT(shader::global::gConf.mValid);
                               return shader::global::gConf; }
#define SHADER_CONF shader::GetConf()
#else
#define SHADER_CONF shader::global::gConf
#endif

//------------------------------------------------------------------------------

} // namespace shader

#endif // SHADER_CONF_HH
