/*****************************************************************************
 * $LastChangedDate: 2010-02-03 18:48:51 -0600 (Wed, 03 Feb 2010) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Module configuration.
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef SKY_CONF_HH
#define SKY_CONF_HH 1

#include "base/conf_txt.hh"
#include "gfx/rgba.hh"
using namespace gfx;
#include "physics/module.hh"
using namespace physics;

namespace sky {

////////////////////////////////////////////////////////////////////////////////
/// @brief Module configuration.
///
class Conf
{
public:
    struct CloudConf
    {
        CloudConf( void )
        : mCloudCount(0), mCloudMinAlt(0), mCloudMaxAlt(0),
          mCloudMinWidth(0), mCloudMaxWidth(0) { }
        uint    mCloudCount;        ///< amount of clouds
        Meter   mCloudMinAlt;       ///< random range
        Meter   mCloudMaxAlt;
        Meter   mCloudMinWidth;     ///< random range
        Meter   mCloudMaxWidth;
    };

private:
    ////////////////////////////////////////////////////////////////////////////
    /// @brief Template Method to parse fields in .conf file.
    ///
    class ModuleConfTxt : public base::ConfTxt
    {
    public:
        void ProcessField( string& field, ifstream& conf );
        void ProcessField( string& field, ifstream& conf, CloudConf& cloudConf );
    };

public:
            Conf( void );
    void    Init( int argc, char** argv, const string& confTxtFilename );

private:
    void    ParseCmdlineArgs( int argc, char** argv );
    void    ParseCmdlineArgs2( int argc, char** argv );
    void    ReadConfTxt( const string& confTxtFilename );

public:
    bool            mValid;     ///< true if Conf was init/read

    // Cmd-line args:
    bool            mSkyEnabled;
    bool            mUseOssim;          ///< WORLD_CONF isn't available to SKY_CONF

    // .conf:
    RGBA            mSkyColor;
    fpx             mSkyFogDensity;     ///< fpx, sky needs lower fog density
    uint            mSkyDomeSliceCount;
    bool            mEnableMoon;
    uint            mStarCount;         ///< amount of stars
    bool            mCloudsEnabled;
    CloudConf       mCloudConfFlat;
    CloudConf       mCloudConfSphere;
    uint            mWeatherRainChance;
    uint            mWeatherSnowChance;
};

ostream& operator<<( ostream& strm, const Conf& conf );

//------------------------------------------------------------------------------

// NOTE: "global" is a namespace, "gGlobals" is a var (instance of Globals class).

namespace global {
#if SKY_MODULE_CC
Conf gConf;
#else
extern Conf gConf;
#endif
}  // namespace global

// SKY_CONF is an abbrevation and also ensures configuration was read in debug build.
#if DEBUG
INLINE Conf& GetConf( void ) { ASSERT(sky::global::gConf.mValid);
                               return sky::global::gConf; }
#define SKY_CONF sky::GetConf()
#else
#define SKY_CONF sky::global::gConf
#endif

//------------------------------------------------------------------------------

} // namespace sky

#endif // SKY_CONF_HH
