/*****************************************************************************
 * $LastChangedDate: 2009-12-14 19:05:10 -0600 (Mon, 14 Dec 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Module configuration.
 *//*
 * LEGAL:   COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define SOUND_CONF_CC 1
#include "base/module.hh"
#include "base/conf.hh"
#include "base/misc.hh"
using namespace base;
#include "sound/module.hh"
#include "sound/conf.hh"

namespace sound {

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////  Conf  ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * ctor/dtor.
 *****************************************************************************/
Conf::Conf( void )
:   mValid(false),
    // cmd-line args
    mEnableSound(true)
    // .conf file
{
    // NOP
}

/*****************************************************************************
 * Initialize/read configuration.
 *****************************************************************************/
void
Conf::Init( int argc, char** argv, const string& confTxtFilename )
{

BEGIN_ONCE {

    mValid = true;  // assign this first as *_CONF may expand to GetConf() which asserts it

    ParseCmdlineArgs( argc, argv );   // cmdline args parsed prior to reading .conf file
    ReadConfTxt( confTxtFilename );   // read from textual .conf file
    ParseCmdlineArgs2( argc, argv );  // cmdline args that override .conf file

} END_ONCE

}

/*****************************************************************************
 * Parse cmdline args.
 *****************************************************************************/
void
Conf::ParseCmdlineArgs( int argc, char** argv )
{
    // NOP
}

void
Conf::ParseCmdlineArgs2( int argc, char** argv )
{
    for ( int i = 1; i < argc; ++i )
    {
        if ( STREQ( argv[i], "-ns" )
          or STREQ( argv[i], "-no-sound" )
          or STREQ( argv[i], "-no-audio" ) )
        {
            mEnableSound = false;
            CLOG << "Disabled sound." << endl;
        }
        else if ( STREQ( argv[i], "-sound" )
               or STREQ( argv[i], "-audio" ) )
        {
            mEnableSound = true;  // doesn't yet mean sound is available
        }
    }
}

/*****************************************************************************
 * Read from .conf file.
 *****************************************************************************/
void
Conf::ReadConfTxt( const string& confTxtFilename )
{
    try
    {
        ModuleConfTxt confTxt;
        confTxt.Read( string(base::GetDataDir()) + string(confTxtFilename) );
    }
    catch (...)
    {
        CERROR << "ERROR: cannot read " << confTxtFilename << endl;
        EXIT( 1 );
    }
}

void
Conf::ModuleConfTxt::ProcessField( string& field, ifstream& conf )
{
    // NOP
}

/*****************************************************************************
 * Print Conf configuration.
 *****************************************************************************/
ostream&
operator<<( ostream& strm, const Conf& conf )
{
    // NOP
    return strm;
}

} // namespace sound
