/*****************************************************************************
 * $LastChangedDate: 2009-12-14 19:05:10 -0600 (Mon, 14 Dec 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Module configuration.
 *//*
 * LEGAL:   COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef SOUND_CONF_HH
#define SOUND_CONF_HH 1

#include "base/conf_txt.hh"

namespace sound {

////////////////////////////////////////////////////////////////////////////////
/// @brief Module configuration.
///
class Conf
{
private:
    ////////////////////////////////////////////////////////////////////////////
    /// @brief Template Method to parse fields in .conf file.
    ///
    class ModuleConfTxt : public base::ConfTxt
    {
    public:
        void ProcessField( string& field, ifstream& conf );
    };

public:
            Conf( void );
    void    Init( int argc, char** argv, const string& confTxtFilename );

private:
    void    ParseCmdlineArgs( int argc, char** argv );
    void    ParseCmdlineArgs2( int argc, char** argv );
    void    ReadConfTxt( const string& confTxtFilename );

public:
    bool            mValid;                ///< true if Conf was init/read
    bool            mEnableSound;
};

ostream& operator<<( ostream& strm, const Conf& conf );

//------------------------------------------------------------------------------

// NOTE: "global" is a namespace, "gGlobals" is a var (instance of Globals class).

namespace global {
#if SOUND_CONF_CC
Conf gConf;
#else
extern Conf gConf;
#endif
}  // namespace global

// SOUND_CONF is an abbrevation and also ensures configuration was read in debug build.
#if DEBUG
INLINE Conf& GetConf( void ) { ASSERT(sound::global::gConf.mValid);
                               return sound::global::gConf; }
#define SOUND_CONF sound::GetConf()
#else
#define SOUND_CONF sound::global::gConf
#endif

//------------------------------------------------------------------------------

} // namespace sound

#endif // SOUND_CONF_HH
