/*****************************************************************************
 * $LastChangedDate: 2009-12-14 20:40:13 -0600 (Mon, 14 Dec 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Sound class (NOP).
 *//*
 * LEGAL:   COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef SOUND_SOUND_NOP_HH
#define SOUND_SOUND_NOP_HH 1

#include "sound/sound_base.hh"
#include "sound/sound_sample.hh"
#include "sound/sound_sample_nop.hh"

namespace sound {

////////////////////////////////////////////////////////////////////////////////
/// @brief Sound class (NOP).
///
/// Class design:
/// -------------
/// See SoundBase.
///
class SoundNOP : public SoundBase
{
PREVENT_COPYING(SoundNOP)
private:
                            SoundNOP( void ) { }
    virtual                 ~SoundNOP() { }

public:
                            DEF_GetInstance( SoundNOP )  // Singleton
    virtual bool            Enable( const bool enable = true ) { return true; }
    virtual bool            IfEnabled( void ) const { return true; }
    virtual fp              GetVolume( void ) const { return 1.0f; }
    virtual bool            SetVolume( const fp volume ) { return true; }
    // [SoundBase defines the other methods]

private:
    CLASS_VAR SoundNOP*     msInstance;
    virtual shptr<Sample>   CreateSample( const Sample::Name& sampleName ) { return new SampleNOP(sampleName); }
};

#if SOUND_MODULE_CC
SoundNOP* SoundNOP::msInstance;
#endif

} // namespace sound

#endif // SOUND_SOUND_NOP_HH
