/*****************************************************************************
 * $LastChangedDate: 2010-01-28 22:56:22 -0600 (Thu, 28 Jan 2010) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Base class for a sound sample.
 *//*
 * LEGAL:   COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef SOUND_SOUND_SAMPLE_HH
#define SOUND_SOUND_SAMPLE_HH 1

#include "sound/defs.hh"

namespace sound {

////////////////////////////////////////////////////////////////////////////
/// @brief Base class for a sound sample.
///
/// Class design:
/// -------------
/// See SoundBase.
///
/// Notes:
/// ------
/// Samples don't know if sound is disabled (responsibility of Sound class).
///
class Sample : public Shared
{
PREVENT_COPYING(Sample)
friend class shptr<Sample>;
public:
    typedef string Name;  ///< filename

protected:
                        Sample( const Sample::Name& sampleName )
                        : mName(sampleName)
                        { SET_TYPESIG(this,TYPESIG_SOUND_SAMPLE); }
    virtual             ~Sample()
                        { INVALIDATE_TYPESIG(this,TYPESIG_SOUND_SAMPLE); }
public:
    const Name          GetName( void ) const { return mName; }
    virtual bool        IfPlaying( void ) const = 0;
    virtual bool        Play( const ePlayLoop playLoop, const fp volume = sound::defs::DEFAULT_VOLUME ) = 0;
    virtual bool        Stop( void ) = 0;
    virtual bool        SetVolume( const fp volume ) = 0;
    virtual fp          GetVolume( void ) const = 0;

private:
    const Name      mName;          ///< filename
public:
    DECL_TYPESIG(TYPESIG_SOUND_SAMPLE)
};

} // namespace sound

#endif // SOUND_SOUND_SAMPLE_HH
