/*****************************************************************************
 * $LastChangedDate: 2010-02-05 15:19:01 -0600 (Fri, 05 Feb 2010) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Module configuration.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef VIEW_CONF_HH
#define VIEW_CONF_HH 1

#include "base/conf_txt.hh"
#include "math/module.hh"
#include "math/types_trig.hh"
#include "math/vertex.hh"
using namespace math;
#include "gfx/module.hh"
#include "gfx/graphics.hh"
using namespace gfx;
#include "object/rotation_list.hh"
using namespace object;

namespace view {

////////////////////////////////////////////////////////////////////////////////
/// @brief Module configuration.
///
class Conf
{
private:
    ////////////////////////////////////////////////////////////////////////////
    /// @brief Template Method to parse fields in .conf file.
    ///
    class ModuleConfTxt : public base::ConfTxt
    {
    public:
        void ProcessField( string& field, ifstream& conf );
    };

public:
            Conf( void );
    void    Init( int argc, char** argv, const string& confTxtFilename );

private:
    void    ParseCmdlineArgs( int argc, char** argv );
    void    ParseCmdlineArgs2( int argc, char** argv );
    void    ReadConfTxt( const string& confTxtFilename );

public:
    bool                mValid;     ///< true if Conf was init/read

    // Cmd-line args:
    // (none)

    // .conf:
    string              mWindowTitle;
    Vector2             mWindowPosition;                ///< top-left corner (if window manager allows)
    WidthHeight<int>    mWindowWidthHeight;
    fpx                 mViewProjectionFOV;             ///< field-of-view degree
    fpx                 mViewProjectionAspectRatio;     ///< field-of-view
    fpx                 mViewNearFarRatio;              ///< affects frustum clipping
    fp                  mOtwViewpointOffset;            ///< to move up viewpoint, mitigate frustum clipping at low alt
    bool                mStatsEnabled;                  ///< controls OSG statistics
    bool                mManipulatorEnabled;            ///< for constructing scene or debugging
    Radian              mManipulatorYawRad;
    Radian              mManipulatorPitchRad;
    fp                  mManipulatorStep;               ///< world space
    Vector3             mManipulatorPosition;           ///< world space
    RotationList        mManipulatorRotationList;       ///< to set orientation
};

ostream& operator<<( ostream& strm, const Conf& conf );

//------------------------------------------------------------------------------

// NOTE: "global" is a namespace, "gGlobals" is a var (instance of Globals class).

namespace global {
#if VIEW_MODULE_CC
Conf gConf;
#else
extern Conf gConf;
#endif
}  // namespace global

// VIEW_CONF is an abbrevation and also ensures configuration was read in debug build.
#if DEBUG
INLINE Conf& GetConf( void ) { ASSERT(view::global::gConf.mValid);
                               return view::global::gConf; }
#define VIEW_CONF view::GetConf()
#else
#define VIEW_CONF view::global::gConf
#endif

//------------------------------------------------------------------------------

} // namespace view

#endif // VIEW_CONF_HH
