/*****************************************************************************
 * $LastChangedDate: 2009-12-07 14:28:30 -0600 (Mon, 07 Dec 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Base class for manipulator (OSG specific).
 *//*
 * LEGAL:   COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef VIEW_MANIPULATOR_HH
#define VIEW_MANIPULATOR_HH 1

#include <osgGA/GUIEventAdapter>
#include <osgGA/GUIActionAdapter>
#include <osgGA/MatrixManipulator>

namespace view {

////////////////////////////////////////////////////////////////////////////////
/// @brief Base class for manipulator (OSG specific).
///
/// A ManipulatorViewpoint can be composed with a Manipulator object.
///
class Manipulator : public osgGA::MatrixManipulator
{
protected:  // don't directly instantiate
                            Manipulator( void ) { }
    virtual                 ~Manipulator() { }

public:
// OSG-specific:
    virtual void            home( double currentTime ) { }
    virtual void            home( const osgGA::GUIEventAdapter& event, osgGA::GUIActionAdapter& action ) { }
    virtual bool            handle( const osgGA::GUIEventAdapter& event, osgGA::GUIActionAdapter& action ) { return false; }
    virtual void            setByMatrix( const osg::Matrixd& matrix ) = 0;
    virtual osg::Matrixd    getMatrix( void ) const = 0;
    virtual void            setByInverseMatrix( const osg::Matrixd& matrix ) = 0;
    virtual osg::Matrixd    getInverseMatrix( void ) const = 0;
};

} // namespace view

#endif // VIEW_MANIPULATOR_HH
