/*****************************************************************************
 * $LastChangedDate: 2010-02-14 12:12:43 -0600 (Sun, 14 Feb 2010) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Module configuration.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef WORLD_CONF_HH
#define WORLD_CONF_HH 1

#include "base/conf_txt.hh"
#include "object/rotation_list.hh"
using namespace object;

namespace world {

////////////////////////////////////////////////////////////////////////////////
/// @brief Module configuration.
///
class Conf
{
private:
    ////////////////////////////////////////////////////////////////////////////
    /// @brief Template Method to parse fields in .conf file.
    ///
    class ModuleConfTxt : public base::ConfTxt
    {
    public:
        void ProcessField( string& field, ifstream& conf );
    };

public:
            Conf( void );
    void    Init( int argc, char** argv, const string& confTxtFilename );

private:
    void    ParseCmdlineArgs( int argc, char** argv );
    void    ParseCmdlineArgs2( int argc, char** argv );
    void    ReadConfTxt( const string& confTxtFilename );

public:
    bool            mValid;                 ///< true if Conf was init/read
    bool            mFlatWorld;             ///< use World::IfFlat() if possible
    bool            mEnableOsgOptimizer;    ///< disabled by -no-osg-opt

    // World:
    fp              mObjectScale;
    Vector3         mObjectScale3;              ///< thrice

    // Terrain model of flat world:
    string          mFlatModelFile;             ///< terrain patch
    RotationList    mFlatModelRotationList;     ///< to set orientation
    fp              mFlatModelScale;            ///< graphical scale
    WorldVertex     mFlatModelOffset;           ///< to adjust origin/height of particular terrain model
    SphereVertex    mFlatModelNW;               ///< NW extent of terrain patch
    SphereVertex    mFlatModelSE;               ///< SE extent of terrain patch
    GeoVertex       mFlatModelMeterOffset;      ///< to adjust center of world
    fp              mFlatModelMeterSimRatio;    ///< multiplying this by a meter converts to graphical coordinates
    Meter           mFlatModelMeterLowAlt;      ///< approx low altitude, for World::IfLowAltitude()
    Meter           mFlatModelMeterAltMin;      ///< for computing volume
    Meter           mFlatModelMeterAltMax;      ///< for computing volume
};

ostream& operator<<( ostream& strm, const Conf& conf );

//------------------------------------------------------------------------------

// NOTE: "global" is a namespace, "gGlobals" is a var (instance of Globals class).

namespace global {
#if WORLD_CONF_CC
Conf gConf;
#else
extern Conf gConf;
#endif
}  // namespace global

// WORLD_CONF is an abbrevation and also ensures configuration was read in debug build.
#if DEBUG
INLINE Conf& GetConf( void ) { ASSERT(world::global::gConf.mValid);
                               return world::global::gConf; }
#define WORLD_CONF world::GetConf()
#else
#define WORLD_CONF world::global::gConf
#endif

//------------------------------------------------------------------------------

} // namespace world

#endif // WORLD_CONF_HH
