/*****************************************************************************
 * $LastChangedDate: 2009-11-22 21:39:11 -0600 (Sun, 22 Nov 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   World definitions.
 *          THESE ARE GLOBALS!  TRY TO USE ONLY BASIC COMPILE-TIME CONSTANTS.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef WORLD_DEFS_HH
#define WORLD_DEFS_HH 1

#include "math/module.hh"
using namespace math;
#include "physics/types.hh"
using namespace physics;

namespace world {
namespace defs {

#define WORLD_COORD(COORD) (COORD)  ///< to scale world coordinates

// Symbolic names for world axises (N/A to matrix axises).
// World axises vs. eye matrix axises:
// In terms of depth, Z defines depth in both world and eye space.
// But in terms of altitude (up/down), Y is up/down in eye space
// while Z is up/down in world space.
enum { AXIS_LAT = AXIS_X, LAT = AXIS_LAT,    // latitude
       AXIS_LON = AXIS_Y, LON = AXIS_LON,    // longitude
       AXIS_ALT = AXIS_Z, ALT = AXIS_ALT };  // altitude

// Macro for constructing a Vector3 independent of definition of axises.
// (For future, this may help to facilitate changing order of axises.)
// WorldVertex pos( LAT_LON_ALT( lat, lon, alt ) );
#define LAT_LON_ALT( LAT, LON, ALT )  (LAT), (LON), (ALT)

// Min/max altitude (for checking).
const Meter MIN_ALTITUDE_METERS( -2000.0f );    ///< Grand Canyon depth is 1.6 KM
const Meter MAX_ALTITUDE_METERS( 100000.f );    ///< SR-71 ceiling is 25,900 meters (16 miles)

// OSG render bins:
const int RENDER_BIN_SKY_DOME = -16;

} // namespace defs
} // namespace world

using namespace world::defs;

#endif // WORLD_DEFS_HH
