/*****************************************************************************
 * $LastChangedDate: 2009-11-22 21:39:11 -0600 (Sun, 22 Nov 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   World class.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef WORLD_WORLD_HH
#define WORLD_WORLD_HH 1

#define GET_WORLD() world::global::gWorld

#include "gfx/module.hh"
#include "gfx/graphics.hh"
using namespace gfx;
namespace object { class Object; }
using namespace object;

namespace world {

////////////////////////////////////////////////////////////////////////////////
/// @brief World class defines the simulated world (excludes scene-graph).
///
/// Historical notes:
/// In previous versions, World served as a facade over the scene-graph,
/// was the container of Objects, etc.  World eventually grew too complex
/// so much of its functionality was factored out to SceneGraph.
/// Currently, World now only has this functionality:
/// - to make the primordial world by World::Make()
/// - provide info about the simulated world
///
class World
{
public:
                    World( void ) { }
                    ~World() { }
    void            Make( void );

// Info:
public:
    WorldVertex     GetPosition( void ) { return WorldVertex(0,0,0); }  // center
    fp              GetRadius( void );
    BoxVolume       GetVolume( void );
    bool            IfFlat( void );
    bool            IfLowAltitude( const WorldVertex& pos );

// Orientation:
public:
    CLASS_CONST int ALT_UP_DIR   = 1;  // altitude
    CLASS_CONST int ALT_DOWN_DIR = -1;
    bool            IfDownward( fp n ) { return n < 0; }
    bool            IfUpward( fp n )   { return n >= 0; }
    WorldVertex     ComputeUpVector( const SphereVertex& sv );
    WorldVertex     ComputeUpVector( Object& object );
    void            RotateTangential( Object& object, const SphereVertex& sv );
    const Matrix&   GetTangentMatrix( const SphereVertex& sv );
};

// Define global.
#if WORLD_MODULE_CC
namespace global { World gWorld; }
#else
namespace global { extern World gWorld; }
#endif

} // namespace world

#endif // WORLD_WORLD_HH
